@@ -3,6 +3,7 @@ from .adapter.factory import AdapterConfigFactory
 from .concepts import Env
 from .core import utils
 from .core.config import Config
+from .core.context import get_config, get_env, set_config
 from .core.envconfig import DotFile, EnvConfig
 from .core.mapping import DEFAULT_MAPPING, EnvMapping
 from .interface import MISSING, ConfigLike, default_cast
@@ -20,4 +21,7 @@ __all__ = [
     "MISSING",
     "default_cast",
     "utils",
+    "get_config",
+    "set_config",
+    "get_env",
 ]
