@@ -8,7 +8,6 @@ from escudeiro.exc.errors import (
     InvalidParamType,
     InvalidPath,
     KeyAlreadyRead,
-    MergingError,
     MissingName,
     RetryError,
     SquireError,
@@ -20,7 +19,6 @@ __all__ = [
     "SquireError",
     "KeyAlreadyRead",
     "InvalidCast",
-    "MergingError",
     "MissingName",
     "AlreadySet",
     "StrictCast",
